clear all
close all
clc
% To observe the matlab demo, type bandem on the command window.
% You will see that 2 different algorithms will go trought different
% minimization paths, that some algorithms don't reach the solution and 
% that the number of steps is different across algorithms.
% The optimization algorithm have to be chosen carefully. However, the
% algorithm BFGS, which is based on the Newton method using the hessian
% matrix, is particularly efficient for the majority of the objective
% functions that you will have to minimize in finance.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fitting a normal law
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

%Choose a mean of 15. Following problem conditions, the three cases for
%the standard deviations are therefore 7.5, 15, and 30. Try more values and
%see what happens. In general, you'll see that the flatter the function
%(i.e., the less concave or convex the objective function) the more problematic the
%determination of the maximum of the log-likelihood.

Values1=normrnd(15,7.5,1000,1);
Values2=normrnd(15,15,1000,1);
Values3=normrnd(15,30,1000,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. + 3.

% We maximize the LOG of the likelihood function to avoid numerical problems in the optimization. 

% Definition of the initial parameters, which are the starting values of
% the optimizer
guesses=[0 1];  % Change these parameters and see what happens! especially, set the second parameter
                 % very close to zero, negative, or very high!. We know that the
                 % second parameter can't be negative, but still putting a
                 % neg. number yields a (very bad) result. So careful with
                 % initial values! Same comment applies for second
                 % parameter too big (i.e., 30)
% We use the fminunc function and we try different algorithms. This is done
% by updating the optimoptions settings.
opt = optimoptions('fminunc','GradObj','on','Algorithm','trust-region');
% to use the trust-region algorithm of fminunc we have to provide Matlab
% with the gradient of the function. This is given as second output of the
% function normloglikelihood.
estimates1=fminunc(@(params) normloglikelihood(params,Values1),guesses,opt)
% Quasi-newton algorithms
opt=optimoptions('fminunc','Display','iter','Algorithm','quasi-newton','HessUpdate','bfgs');
% 'HessUpdate':
% 'bfgs', the default
% 'dfp'
% 'steepdesc'
estimates1=fminunc(@(params) normloglikelihood(params,Values1),guesses,opt)
% fminsearch
opt=optimset('Display','iter')
estimates1=fminsearch(@(params) normloglikelihood(params,Values1),guesses,opt)
% change the starting values
guesses=[8 10]
estimates1=fminsearch(@(params) normloglikelihood(params,Values1),guesses,opt)

% Comments:
% - you should observe that the higher is the variance, the less performant
% is the optimizer.
% - for some initial values, the optimizer doesn't converge towards the
% solution.

% fminsearch uses the simplex method
% fminunc uses the gradient -> for smooth functions always use fminunc!!!



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 4.

% In the case of a gaussian law, the maximization of the log-likelihood 
% admits an explicit solution and it is not necessary to compute numeric optimizations.
% The explicit solution is obtained by zeroeing the gradient, i.e.
% calculate the partial derivatives of the log-likelihood with respect to
% the mean and variance and look for the parameters value that make the 
% derivatives be equal to zero. 
% The solution for the mean is the empirical estimator of the mean.
% The solution for the variance is the empirical estimator (1/N) of the
% variance.

estimatesexplicites1=[mean(Values1) var(Values1,1)]
estimatesexplicites2=[mean(Values2) var(Values2,1)]
estimatesexplicites3=[mean(Values3) var(Values3,1)]

% In practice, if you calculate these explicit solutions and you compare
% them with the solutions obtained with the numeric optimization, you
% should see some differences.
% Moral: when is possible, always use the explicit solution, otherwise
% adjust at best the optimizer.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 5.
% Use of the normfit function
[matlabmean matlabvol]=normfit(Values1);
[mleparam]=mle(Values1);
estimatesnormfit=[matlabmean matlabvol^2]

% You can see that matlab with mle calculates the explicit solution.
% Note also that normfit gives as output the square root of the unbiased
% estimator for the variance


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimation of a non linear contditional mean
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load the data
load 'TP5.mat' % load the datafile, already in Matlab format
N=length(X1);
bandwidth=(N^(-0.2))*(var(X1)^0.5)/2;
condmean1=condmean_kernel(Y1,X1,200,bandwidth);
condmean2=condmean_kernel(Y2,X2,200);

figure
plot(condmean1(1,:),condmean1(2,:),condmean1(1,:),...
    0.03*sin(pi*condmean1(1,:)/90)+2e-4*condmean1(1,:))
title('Estimation of the conditional mean by gaussian kernel - Set 1')
xlabel('Value of the factor (x)')
ylabel('Value of the conditional mean')
legend('Estimated','True DGP')
figure
plot(condmean2(1,:),condmean2(2,:),condmean2(1,:),0.002*condmean2(1,:));
title('Estimation of the conditional mean by gaussian kernel - Set 2')
xlabel('Value of the factor (x)')
ylabel('Value of the conditional mean')
legend('Estimated','True DGP')

% They are not financial data. The series are generated by:
% - serie 1: Y1=0.03*sin(pi*X1/90)+2e-4*X1+errors, with normal errors 
% - serie 2: Y2=0.002*X2+erreurs, 



